/*
 * Copyright 2005 - 2011  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 *
 * @author S.B. Kok
 * @version 1.0
 *
 * This Javascript file extracts useful information from the
 * Zarafa WebAccess environment, such that the Spreed Plugin
 * can be started with the contents of mails, contacts, etc.
 * 
 */
Pluginspreed.prototype = new Plugin;
Pluginspreed.prototype.constructor = Pluginspreed;
Pluginspreed.superclass = Plugin.prototype;

/**
 * constructor
 */
function Pluginspreed() {}

/**
 * Pluginspreed init method
 * This method will register all the hooks that are
 * necessary to hook the buttons in the Zarafa WebAccess
 * interface.
 */
Pluginspreed.prototype.init = function(){
    this.registerHook("client.module.contactlistmodule.contextmenu.buildup");
    this.registerHook("client.module.maillistmodule.contextmenu.buildup");
    this.registerHook("client.module.readmailitemmodule.recipient.contextmenu.buildup");
    this.registerHook("client.module.maillistmodule.topmenu.buildup");
    this.registerHook("client.module.contactlistmodule.topmenu.buildup");
    this.registerHook("client.core.webclient.init.after");
}

/**
 * Pluginspreed execute
 * It is called when registered events of Pluginspreed are triggered,
 * all triggered from the Zarafa WebAccess interface.
 */
Pluginspreed.prototype.execute = function(eventID, data){
	switch(eventID) {
        case "client.module.contactlistmodule.topmenu.buildup":
			this.addContextMenuItem(data["topmenu"], false);
			break;
		case "client.module.contactlistmodule.contextmenu.buildup":
			this.addContextMenuItem(data["contextmenu"], false);
			break;
        case "client.module.readmailitemmodule.recipient.contextmenu.buildup":
            /* Intentional fall-through */
        case "client.module.maillistmodule.contextmenu.buildup":
			this.addContextMenuItem(data["contextmenu"], true);
			break;
        case "client.module.maillistmodule.topmenu.buildup":
			this.addContextMenuItem(data["topmenu"], true);
            break;
        case "client.core.webclient.init.after":
            /*
             * Since the default menu items are generated directly at the start, we
             * need to add our button directly after the init function of the webclient
             * has returned. Therefore we wait for this hook to fire.
             */
            this.addDefaultMenuItem();
            break;
	}
}

/**
 * Add the Spreed web-meeting button to the default menu that
 * lists all the options to create new emails, contacts, etc.
 */
Pluginspreed.prototype.addDefaultMenuItem = function(){
	// Add separator and "Spreed Meeting" menubutton in the default menu of the webclient.
	webclient.menu.defaultMenuItems.push(webclient.menu.createMenuItem("seperator", ""));
    webclient.menu.defaultMenuItems.push(webclient.menu.createMenuItem("spreed_call", dgettext('plugin_spreed', "New"), dgettext("plugin_spreed", "Spreed Meeting"), eventPluginspreedListMenuSetupEmptyCall));
}


/**
 * Function which adds context menuitem in contextmenu
 * of the given menu structure. This is called to add the
 * Spreed web-meeting button in the context-menu of emails
 * contacts, and distribution lists.
 * @param mixed data object data array of context menu menuitems 
 * @param boolean isMail Set to true if this is a mail we are
 * right-clicking on. Otherwise it is the contact interface.
 */
Pluginspreed.prototype.addContextMenuItem = function(data, isMail){
	// Add separator and "Spreed Meeting" menubutton in context menu of contactlist module, maillist module, top menu, etc.
	data.push(webclient.menu.createMenuItem("seperator", ""));
    data.push(webclient.menu.createMenuItem("spreed_call", dgettext("plugin_spreed", "Spreed Meeting"), dgettext("plugin_spreed", "Set-up Spreed Meeting"), isMail ? eventPluginspreedListMailMenuSetupCall : eventPluginspreedListContactMenuSetupCall));
}

/**
 * Extract the name from the display name string.
 * The display name string can be in many formats, but the
 * most common format is either:
 * Firstname Lastname (Organisation)
 * or
 * Lastname, Firstname (Organisation)
 * Based on these two formats this function is able to extract
 * the firstname, lastname and organisation, and return that
 * as the result string array.
 * @param string fullname The display string that contains the
 * name information that you want to extract.
 * @return string[] The array with the fields: firstname,
 * lastname and organisation.
 */
Pluginspreed.prototype.extractNameArrayFromNameField = function(fullname)
{
    var retArray = Array();
    var names = Array();
    var mode = '';
    retArray['firstname'] = retArray['lastname'] = retArray['organisation'] = retArray['email'] = '';

    fullname = fullname.trim();

    var commaPos = fullname.indexOf(',');
    var orgPos = fullname.indexOf('(');

    // Is the name provided in the following format? Doe, John
    if(commaPos !== -1){
        // Case: The last name is the first part
        // Example: Doe, John (Zarafa)

        // This is the easy part, the comma deliminates the first
        // and lastnames, which is perfect, we do not need to check
        // where the firstname ends and the lastname start here...

        // Set the lastname first, this is the part until the first comma
        retArray['lastname'] = fullname.substring(0, commaPos).trim();

        // Is the organisation name available?
        if(orgPos !== -1){
            retArray['firstname'] = fullname.substring(commaPos + 1, orgPos - 1).trim();
            retArray['organisation'] = fullname.substring(orgPos + 1, fullname.indexOf(')') - 1).trim();
        } else {
            retArray['firstname'] = fullname.substring(commaPos + 1).trim();
        }
    } else {
        // Case: The first name is the first part
        // Example: John Doe (Zarafa)

        // Is the organisation name available?
        if(orgPos !== -1){
            names = fullname.substring(0, orgPos).trim().split(' ');
            retArray['organisation'] = fullname.substring(orgPos + 1, fullname.indexOf(')') - 1).trim();
        } else {
            names = fullname.split(' ');
        }

        // Loop through the names until you find a lower case
        // name, such as van, der, etc. and add those to the first name
        // field. All the other names will be appended to the last name.
        mode = 'firstname';
        for(var i = 0; i < names.length; i++){
            // If we are in firstname mode and we find a name such as van
            // der, etc. or we reached the last name in the name string
            // we should switch to lastname mode and append all the other
            // names to the lastname instead.
            if(mode == 'firstname' && (/^[a-z]/.exec(names[i]) != null || i == (names.length - 1))){
                mode = 'lastname';
                retArray[mode] = names[i];
            } else if(i == 0) {
                retArray[mode] = names[i];
            } else {
                retArray[mode] += ' ' + names[i];
            }
        }
        retArray['firstname'] = retArray['firstname'].trim();
        retArray['lastname'] = retArray['lastname'].trim();
    }

    return retArray;
}

/**
 * Extract the name from the email header.
 * This is used to process the email address and extract
 * the firstname, lastname and organisation if possible.
 * @param string mailHeader The display string that contains the
 * name information that you want to extract.
 * @return string[] The array with the fields: firstname,
 * lastname, organisation and email.
 */
Pluginspreed.prototype.extractNameArrayFromEmailHeader = function(mailHeader)
{
    mailHeader = mailHeader.trim();
    var retArray = Array();
    var lessThanPos = mailHeader.indexOf('<');
    if(lessThanPos !== -1){
        // This email address has a name specified
        retArray = this.extractNameArrayFromNameField(mailHeader.substring(0, lessThanPos));
        retArray['email'] = mailHeader.substring(lessThanPos + 1, mailHeader.length - 1);
    } else {
        // This is just a plain email address, parse it differently
        retArray['firstname'] = retArray['lastname'] = retArray['organisation'] = '';
        retArray['email'] = mailHeader;

        matches = Array();
        if((matches = /^([a-zA-Z0-9._%+-]+)@([a-zA-Z0-9.-]+\.[a-zA-Z]{2,4})$/.exec(mailHeader)) != null){
        
            var user = matches[1];
            retArray['organisation'] = matches[2].trim();

            var flmatches = Array();
            if((flmatches = /(.+)(\-|\.|\_)(\w+)$/.exec(user)) != null){
                retArray['firstname'] = flmatches[1].trim();
                retArray['lastname'] = flmatches[3].trim();
            } else {
                retArray['lastname'] = user.trim();
            }
        }
    }
    return retArray;
}

/**
 * In case the user selected multiple emails or contacts in
 * the Zarafa WebAccess interface, we need to determine their
 * entry ids, to acquire these this function is called.
 * @param string[] entries The list with all the entries inside.
 * @param int[] ids The list with ids that should be selected,
 * each entry in this list represents the index of entries that
 * should be returned.
 * @return string[] The list with the entry ids.
 */
Pluginspreed.prototype.getEntryIDsInArray = function(entries, ids){
    var i, ret;
    ret = [];

    if(ids.constructor != Array)
        ids = [ids];

    for(i = 0; i < ids.length; i++){
        ret.push(entries[ids[i]]);
    }

    return ret;
}

/**
 * Function which opens dialogbox to setup the conference call with Spreed
 * if the user clicked a mail related Spreed button.
 * @param moduleObject The module reference.
 * @param element The element on which the user clicked.
 * @param even The event object container.
 */
function eventPluginspreedListMailMenuSetupCall(moduleObject, element, event) {
    var rowelement, selection, actions, mails, entryids, i, tmp, uri, mailaddresses, recipients, content, close;

    nwTitle = nwDescription = nwParticipants = null;
    actions = [];
    mails = [];
    close = false;

    // Close the context menu if it is still open
	if(element.parentNode.elementid)
        element.parentNode.style.display = "none";

    if(window.module && window.module.getModuleName() == "readmailitemmodule"){
        var props = window.module.itemProps;
        actions.push({ 'data' : props['subject'], 'action' : 'setTitle'});
        // TODO Implement the description as well:
        
        // Get the content of the message body:
        content = props['body'];
        // Replace paragraphs and breaks for new lines. Also replace all entity-spaces for real spaces
        content = content.replace(/<[br|p]\/?>/g, "\n").replace(/&nbsp;/g, " ");
        // Replace all the CSS Style definitions, the XHTML comments, and all the tags that surrounds text.
        content = content.replace(/<style type="text\/css">[^<]+<\/style>/g, '').replace(/<!--[^>]*-->/g, "").replace(/<\/?\w+((\s+\w+(\s*=\s*(?:".*?"|'.*?'|[^'">\s]+))?)+\s*|\s*)\/?>/g, "");

        actions.push({ 'data' : content.trim(), 'action' : 'setDescription'});

        //mailaddresses = Array();
        //mailaddresses.push(props['sender_name'] + " <" + props['sender_email_address'] + ">");
        //for(i = 0; i < props['recipients']['recipient'].length; i++){
        //    mailaddresses.push(props['recipients']['recipient'][i]['display_name'] + " <" + props['recipients']['recipient'][i]['email_address'] + ">");
        //}

        //actions.push({ 'data' : mailaddresses, 'action' : 'addParticipantEmails'});

        mails.push({
                store : props['store_entryid'],
                entryid : props['entryid']
            });

        // Since the user is switching to Spreed, close the current read window
        // so it can be replaced by the new dialog of the Spreed Plugin.
        close = true;
    } else if(window.module && window.module.getModuleName() == "createmailitemmodule"){
        actions.push({ 'data' : dhtml.getElementById("subject").value, 'action' : 'setTitle'});
        // TODO Implement the description as well:
        
        // Get the content of the message body:
        if (typeof FCKeditorAPI != "undefined" && (fckEditor = FCKeditorAPI.GetInstance("html_body")) && fckEditor.GetXHTML() != null){
            // Check for HTML format
            content = fckEditor.GetXHTML();
            // Replace paragraphs and breaks for new lines. Also replace all entity-spaces for real spaces
            content = content.replace(/<[br|p]\/?>/g, "\n").replace(/&nbsp;/g, " ");
            // Replace all the CSS Style definitions, the XHTML comments, and all the tags that surrounds text.
            content = content.replace(/<style type="text\/css">[^<]+<\/style>/g, '').replace(/<!--[^>]*-->/g, "").replace(/<\/?\w+((\s+\w+(\s*=\s*(?:".*?"|'.*?'|[^'">\s]+))?)+\s*|\s*)\/?>/g, "");
        } else {
            // Check for PLAIN format
            content = dhtml.getElementById("html_body").value.trim();
        }

        actions.push({ 'data' : content.trim(), 'action' : 'setDescription'});

        mailaddresses = Array();
        tmp = dhtml.getElementById("to").value + ";" + dhtml.getElementById("cc").value + ";" + dhtml.getElementById("bcc").value;
        recipients = tmp.split(';');
        for(i = 0; i < recipients.length; i++){
            tmp = recipients[i].trim();
            if(tmp.length > 0){
                mailaddresses.push(tmp);
            }
        }
        actions.push({ 'data' : mailaddresses, 'action' : 'addParticipantEmails'});


        mails.push({
                store : window.module.storeid,
                entryid : window.module.messageentryid
            });

        // Mhm, the user made up his or her mind, make sure that all the recently uploaded files are moved
        // from this dialog window to the Spreed Dialog.
        actions.push({ 'data' : { 'from' : dhtml.getElementById("dialog_attachments").value, 'to' : ''}, 'action' : 'moveUploads' });

        // Since the user is switching to Spreed, close the current compose window
        // so it can be replaced by the new dialog of the Spreed Plugin.
        close = true;
    } else if(moduleObject){ 
        // It's the top menu, check 
        selection = moduleObject.getSelectedMessages();
        if(selection.length > 0){
            entryids = Pluginspreed.prototype.getEntryIDsInArray(moduleObject.entryids, moduleObject.getSelectedMessages());
            for(i = 0; i < entryids.length; i++){
                mails.push({
                        'store' : moduleObject.storeid,
                        'entryid' : entryids[i]
                    });
            }
        } else {
            alert(dgettext('plugin_spreed', "Select at least one mail to start a Spreed Meeting with those users."));
        }
    } else if(element.parentNode.elementid){
        // Check whether this event was fired from a context menu
        // or the top-menu.

        // It's a context menu, get the selected element:
        rowelement = dhtml.getElementById(element.parentNode.elementid);
        mails.push({
                store : moduleObject.storeid,
                entryid : moduleObject.entryids[rowelement.id]
            });
    } else {
        console.log("I don't know what to do with this spreed click event, where is it coming from?");
        if(window.module)
            console.log("Module name" + window.module.getModuleName());
        return;
    }   

    if(mails.length > 0){
        actions.push({ 'data' : mails, 'action' : 'getMails'});
    }

	uri = DIALOG_URL + "task=spreed&plugin=spreed";
	webclient.openWindow(moduleObject, 'spreed', uri, 820, 470, true, null, null, { 'actions' : actions });

    // Close the window if neccessary, for example when the user was composing a mail at first, but
    // chose to switch to a Spreed Meeting instead.
    if(close && false)
        window.close();
}

/**
 * Function which opens dialogbox to setup the conference call with Spreed
 * if the user clicks on the new Spreed Call button
 * @param moduleObject The module reference.
 * @param element The element on which the user clicked.
 * @param even The event object container.
 */
function eventPluginspreedListMenuSetupEmptyCall(moduleObject, element, event){
    if(element.parentNode.id == "defaultmenu") {
        element.parentNode.style.display = "none";
    }

	uri = DIALOG_URL + "task=spreed&plugin=spreed";
	webclient.openWindow(moduleObject, 'spreed', uri, 820, 470, true);
}

/**
 * Function which opens dialogbox to setup the conference call with Spreed
 * if the user clicks a contact related Spreed button
 * @param moduleObject The module reference.
 * @param element The element on which the user clicked.
 * @param even The event object container.
 */
function eventPluginspreedListContactMenuSetupCall(moduleObject, element, event) {
    var rowelement, selection, actions, contacts, getcontacts, entryids, i, j, firstAddress, tmpMail, participant, tmp, uri, mailaddresses, recipients, content, close, nwTitle, nwDescription, nwParticipants;

    nwTitle = nwDescription = nwParticipants = null;
    actions = [];
    contacts = [];
    getcontacts = [];
    close = false;

    // Close the context menu if it is still open
	if(element.parentNode.elementid)
        element.parentNode.style.display = "none";

    if(window.module && window.module.getModuleName() == "contactitemmodule"){
        tmp = Pluginspreed.prototype.extractNameArrayFromNameField(dhtml.getElementById('display_name').value);
        tmp['email'] = dhtml.getElementById('email_address').value;
        actions.push({ 'data' : Array(tmp), 'action' : 'addParticipantsJS'});

        // Since the user is switching to Spreed, close the current compose window
        // so it can be replaced by the new dialog of the Spreed Plugin.
        close = true;
    } else if(moduleObject){ 
        // It's the top menu, check 
        selection = moduleObject.getSelectedMessages();
        if(selection.length > 0){
            entryids = Pluginspreed.prototype.getEntryIDsInArray(moduleObject.entryids, moduleObject.getSelectedMessages());
            for(i = 0; i < entryids.length; i++){
				var props = moduleObject.itemProps[entryids[i]];
                
                if(props){
                    if(props['message_class'] == "IPM.DistList"){
                        getcontacts.push({
                                    'entryid' : props['entryid'],
                                    'store' : moduleObject.storeid
                                });
                    } else {
                        firstAddress = true;
                        for(j = 1; j <= 3; j++){
                            tmpMail = props["email_address_" + j] ? props["email_address_" + j].trim() : '';
                            if(tmpMail.length > 0){
                                participant = Pluginspreed.prototype.extractNameArrayFromEmailHeader(tmpMail);
                                
                                // Check if we can use the full name to set the first, last and organisation names
                                if(props["full_name"] && props["full_name"].trim().length > 0){
                                    tmp = Pluginspreed.prototype.extractNameArrayFromNameField(props["full_name"] ? props["full_name"] : '');
                                    
                                    if(tmp['firstname'].length > 0)
                                        participant['firstname'] = tmp['firstname'];
                                    if(tmp['lastname'].length > 0)
                                        participant['lastname'] = tmp['lastname'];
                                    if(tmp['organisation'].length > 0)
                                        participant['organisation'] = tmp['organisation'];
                                }

                                if(props["given_name"].trim().length > 0){
                                    participant['firstname'] = props["given_name"];
                                }
                                if(props["surname"].trim().length > 0){
                                    participant['lastname'] = (props["middle_name"].trim().length > 0 ? props["middle_name"] + ' ' : '') + props["surname"];
                                }
                                if(props["company_name"] && props["company_name"].trim().length > 0){
                                    participant['organisation'] = props["company_name"].trim();
                                }
                                participant['alternative'] = !firstAddress;
                                firstAddress = false;

                                contacts.push(participant);
                            }
                        }
                    }
                }
            }
        } else {
            alert(dgettext('plugin_spreed', "Select at least one contact to start a Spreed Meeting with those users."));
        }
    } else if(window.module && (window.module.getModuleName() == "distlistmodule" || window.module.getModuleName() == "namedistlistmodule")){
        content = dhtml.getElementById('items');

        mailaddresses = Array();
        for(i = 0; i < content.rows.length; i++){
            tmp = Pluginspreed.prototype.extractNameArrayFromNameField(content.rows[i].childNodes[1].textContent);
            tmp['email'] = content.rows[i].childNodes[2].textContent;
            mailaddresses.push(tmp);
        }

        actions.push({ 'data' : mailaddresses, 'action' : 'addParticipantsJS'});

        // Since the user is switching to Spreed, close the current compose window
        // so it can be replaced by the new dialog of the Spreed Plugin.
        close = true;
    } else if(element.parentNode.elementid){
        // Check whether this event was fired from a context menu
        // or the top-menu.

        // It's a context menu, get the selected element:
        rowelement = dhtml.getElementById(element.parentNode.elementid);
        contacts.push({
                store : moduleObject.storeid,
                entryid : moduleObject.entryids[rowelement.id]
            });
    } else {
        console.log("I don't know what to do with this spreed click event, where is it coming from?");
        if(window.module)
            console.log("Module name " + window.module.getModuleName());
        return;
    }   

    if(contacts.length > 0){
        actions.push({ 'data' : contacts, 'action' : 'addParticipantsJS'});
    }
    if(getcontacts.length > 0){
        actions.push({ 'data' : getcontacts, 'action' : 'getDistLists'});
    }

	uri = DIALOG_URL + "task=spreed&plugin=spreed";
	webclient.openWindow(moduleObject, 'spreed', uri, 820, 470, true, null, null, { 'actions' : actions });

    // Close the window if neccessary, for example when the user was composing a mail at first, but
    // chose to switch to a Spreed Meeting instead.
    if(close && false)
        window.close();
}

