/*
 * Copyright 2005 - 2011  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 *
 * @author S.B. Kok
 * @version 1.0
 *
 * This Javascript file processes the user interface events that
 * are fired by the Spreed plugin buttons that provide interaction
 * with the Spreed plugin from within other Zarafa WebAccess
 * components.
 *
 */

/**
 * Function which will setup the conference call for the user
 */
function eventStartCallClicked()
{
	if(module)
        module.setupCall();
}

/**
 * Function which will open up the attachment dialog so the user
 * can upload new attachments manually.
 */
function eventAttachFilesClicked()
{
	if(module)
        module.openAttachmentDialog();
}


/**
 * Function which will open up the address book so the user
 * can add contacts from the address book.
 */
function eventAddressBookClicked()
{
	if(module)
        module.openAddressBook();
}


/**
 * Clear the Spreed Meeting details so the user could
 * start over.
 */
function eventClearClicked()
{
	if(module)
        module.clear();
}

/**
 * Process the event in which the user changed the timezone of
 * of the meeting, such that other selection boxes will change
 * as well.
 */
function eventSpreedTimezoneChanged()
{
    if(module)
        module.timezoneChanged();
}

/**
 * Process the event where the user pressed the return
 * key, so that the user is able to add multiple participants
 * using the form quickly.
 * @param inst The instance of the module
 * @param element The element that fired the event
 * @param e The event instance, used to determine which key
 * was pressed.
 **/
function eventSpreedKeyPressedInInput(inst, element, e)
{
    if(!module && !inst)
        return true;
    else if(!module)
        module = inst;

    var keyCode = e ? (e.which ? e.which : e.keyCode) : event.keyCode;
    if (keyCode == 13) { 
        // Return key is pressed
        module.addParticipantInput();
    }
    return true;
}

/**
 * Process the call back from the address book and add the
 * users to the Spreed Meeting.
 * @param recips The recipients to add.
 */
function abCallBackSpreed(recips)
{
    if(module)
        module.addParticipantEmails(recips.value.split(';'));
}
