/*
 * Copyright 2005 - 2011  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 *
 * @author S.B. Kok
 * @version 1.0.1
 *
 * This Javascript handles all the user interface actions of
 * the Spreed Plugin. These actions include retrieving data
 * from emails and contacts by communicating with the server,
 * but also processing the final data to set-up the Spreed
 * web meeting.
 * 
 */
spreeditemmodule.prototype = new ItemModule;
spreeditemmodule.prototype.constructor = spreeditemmodule;
spreeditemmodule.superclass = ItemModule.prototype;

/**
 * Constructor 
 */ 
function spreeditemmodule(id)
{
	if(arguments.length > 0) {
		this.init(id);
	}
}

/**
 * Initialization
 */ 
spreeditemmodule.prototype.init = function(id) {
	spreeditemmodule.superclass.init.call(this, id);
	/**
	 * Properties array which stores address, latitude and longitude of resolved location
	 * And then it is used for sending caching request to the server.
	 */
    this.checkinURL = null;
    this.creatorRowID = false;
    this.selectCheckIsBusy = false;
    this.startTime = new timePicker(dhtml.getElementById("start_time"), '');
    this.endTime = new timePicker(dhtml.getElementById("end_time"), '');
   
    /*
     * Ceil the current time with five minutes
     */ 
    var nwDate = new Date(new Date().ceilMinutes(5));//.addUnixTimeStampToDate(300);
    //this.startTime.setValue((nwDate.getTime() / 1000) - timeToZero(nwDate.getTime() / 1000));
    //this.endTime.setValue((nwDate.getTime() / 1000) + 1800 - timeToZero(nwDate.getTime() / 1000));

    this.combinedTime = new combinedDateTimePicker(this.startTime, this.endTime);
    this.combinedTime.setStartValue((nwDate.getTime() / 1000) - timeToZero(nwDate.getTime() / 1000));
    this.combinedTime.setEndValue((nwDate.getTime() / 1000) + 1800 - timeToZero(nwDate.getTime() / 1000));

    // Clone the timezone select box
    var nwTimezone = this.makeSelectClone(dhtml.getElementById('timezone'), 'newtimezone');
    dhtml.getElementById("newtimezonetd").appendChild(nwTimezone);
}

/**
 * Function which execute an action. 
 *
 * @param string type the action type
 * @param object action the action tag 
 */ 
spreeditemmodule.prototype.execute = function(type, action)
{
	switch(type){
		case "item":
			//this.item(action);
			break;
		case "getAttachments":		// Uploaded Attachment list.
            // TODO Implement this
			//this.attachmentsList(action);
			break;
        case "getMails":
            // This is an internal JS event, obtained through the
            // window data object. Forward the request to the server.
            this.getEntriesFromServer(action, 'getMail');
            break;
        case "getDistLists":
            // This is an internal JS event, obtained through the
            // window data object. Forward the request to the server.
            this.getEntriesFromServer(action, 'getDistList');
            break;
        case "setTitle":
            // This is an internal JS event, obtained through the
            // window data object. 
            this.setTitle(action);
            break;
        case "setDescription":
            // This is an internal JS event, obtained through the
            // window data object. 
            this.setDescription(action);
            break;
        case "moveUploads":
            // This is an internal JS event, obtained through the
            // window data object. 
            this.moveUploads(action);
            break;
        case "spreedCheckin":
            this.spreedCheckin(action);
            break;
        case "addParticipantsJS":
            // This is an internal JS event, obtained through the
            // window data object. 
            this.addParticipants(action);
            break;
        case "addParticipants":
            // This is the server response on the request to getMails.
            this.addParticipantXMLList(action.getElementsByTagName('user'));
            break;
        case "addParticipantEmails":
            // This is an internal JS event, obtained through the
            // window data object. 
            this.addParticipantEmails(action);
            break;
        case "addAttachments":
            // This is the server response on the request to getMails.
            this.addMailAttachments(action.getElementsByTagName('attachment'));
            break;
	}
}

/**
 * This function will notify the server that all the uploads
 * in the current upload session should be moved to the
 * Spreed web meeting upload session. This is used in case
 * the user made up his/her mind to switch from composing
 * a mail to setting-up a Spreed meeting.
 * @param mixed action The action attribute variable that
 * contains the reference to the current upload session.
 */
spreeditemmodule.prototype.moveUploads = function(action)
{
    action.to = dhtml.getElementById("dialog_attachments").value;

    webclient.xmlrequest.addData(this, "moveUploads", action);
    webclient.xmlrequest.sendRequest();
}

/**
 * Set the title to the specified value.
 * @param string title The new title to use.
 */
spreeditemmodule.prototype.setTitle = function(title)
{
    dhtml.getElementById("title").value = escapeHtml(title);
}

/**
 * Set the description to the specified value.
 * @param string description The new description to use.
 */
spreeditemmodule.prototype.setDescription = function(description)
{
    dhtml.getElementById("description").value = escapeHtml(description);
}

/**
 * If the server was told to retrieve one or more mails for
 * the Spreed web meeting, this function will process all the
 * attachments that were gathered by the server. These will be
 * made visible to the user, after which the user can decide
 * to remove the uploads using the Attachment Dialog if necessary.
 */
spreeditemmodule.prototype.addMailAttachments = function(list){
    var attachmentsElement = dhtml.getElementById("itemattachments");
    var attachments = Array();

    if(!attachmentsElement) return;

    for(i = 0; i < list.length; i++){
        var attach = Array();
        attach['attach_num'] = dhtml.getXMLValue(list[i], "attach_num"); 
        attach['name'] = escapeHtml(dhtml.getXMLValue(list[i], "name"));
        attach['size'] = dhtml.getXMLValue(list[i], "size");
        attach['filetype'] = dhtml.getXMLValue(list[i], "filetype");
        attachments.push(attach);
    }

    this.addAttachments(attachmentsElement, attachments);
}

/**
 * This function is called once the server is ready with
 * setting up the conference meeting, it will show the button
 * and instructs the user to either close the window or
 * start the conference meeting right away.
 * @param mixed action The return values we received from
 * the server after it set-up the meeting.
 **/
spreeditemmodule.prototype.spreedCheckin = function(action)
{
    var err = dhtml.getXMLValue(action, 'error');
    if(err){
        dhtml.addClassName(dhtml.getElementById('overlay'), 'hidden');
        alert(err);
        return;
    }

    dhtml.addClassName(dhtml.getElementById('icon_loading'), 'hidden');
    dhtml.removeClassName(dhtml.getElementById('startConfButton'), 'hidden');
    dhtml.getElementById('overlayText').innerHTML = dgettext('plugin_spreed', "Spreed Meeting is ready, click the button below to start right away") + ".";
    this.checkinURL = dhtml.getXMLValue(action, 'checkinURL');
}

/**
 * Handle the onClick event of the Start Meeting button,
 * this will open up a new window with the spreed-checkin URL.
 **/
spreeditemmodule.prototype.openConferenceMeeting = function()
{
    if(this.checkinURL)
        window.open(this.checkinURL, 'Spreed_Meeting');
}

/**
 * create and set-up the participants table in the module.
 * this function is called once the spreed plugin window is
 * opened to set-up the participant table, with the user
 * at the first row of the table by default.
 * @param firstname the first name of the user.
 * @param lastname the surname of the user.
 * @param organisation the organisation of the user.
 * @param email the email address of the user.
 */
spreeditemmodule.prototype.createParticipantsTable = function(firstname, lastname, organisation, email)
{
    // create a list of the subfolders using the tablewidget
	var tableWidgetElem = dhtml.addElement(dhtml.getElementById("spreedparticipants"), "div", false, "tableWidgetContainer");
	//tablewidgetelem.style.height = "300px";

	this.participants_tableWidget = new TableWidget(tableWidgetElem, true);
    this.setupParticipantsTable();

    this.addCreatorToTable(firstname, lastname, organisation, email);
}

/**
 * create and set-up the participants table in the module.
 * this function is called once the spreed plugin window is
 * opened to set-up the participant table, with the user
 * at the first row of the table by default.
 * @param firstname the first name of the user.
 * @param lastname the surname of the user.
 * @param organisation the organisation of the user.
 * @param email the email address of the user.
 */
spreeditemmodule.prototype.addCreatorToTable = function(firstname, lastname, organisation, email)
{
    var st, e;
    
    if(firstname !== false && !this.creator){
        this.creator = Array();
        this.creator['firstname'] = firstname;
        this.creator['lastname'] = lastname;
        this.creator['organisation'] = organisation;
        this.creator['email'] = email;
    }
    this.creatorRowID = this.addParticipant(this.creator['firstname'], this.creator['lastname'], this.creator['organisation'], this.creator['email'], true);


    // Disable the select boxes 
    dhtml.getElementById('inc_' + this.creatorRowID).setAttribute('disabled', 'disabled');
    dhtml.getElementById('l_' + this.creatorRowID).setAttribute('disabled', 'disabled');
    dhtml.getElementById('tz_' + this.creatorRowID).setAttribute('disabled', 'disabled');

    // Remove the edit button
    e = dhtml.getElementById('e_' + this.creatorRowID);
    e.setAttribute('onclick', '');
    e.setAttribute('style', 'visibility: hidden;');

    // Set the status to moderator
    st = dhtml.getElementById('st_' + this.creatorRowID);
    st.setAttribute('alt', 'm');
    st.setAttribute('onclick', '');
    st.setAttribute('title', dgettext('plugin_spreed', 'You are the moderator of the meeting') + '.');
    st.className = 'icon_spreed_mod';
}

/**
 * Setup the participants table in the module, this will add
 * the columns to the table and make sure it is generated.
 */
spreeditemmodule.prototype.setupParticipantsTable = function()
{
	this.participants_tableWidget.addColumn("include", "", 20, 1);
	this.participants_tableWidget.addColumn("firstname", dgettext('plugin_spreed', "Firstname"), 80, 1);
	this.participants_tableWidget.addColumn("lastname", dgettext('plugin_spreed', "Surname"), 100, 2);
	this.participants_tableWidget.addColumn("organisation", dgettext('plugin_spreed', "Organisation"), 110, 3);
	this.participants_tableWidget.addColumn("email", dgettext('plugin_spreed', "Email"), 200, 4);
	this.participants_tableWidget.addColumn("language", dgettext('plugin_spreed', "Language"), 120, 5);
	this.participants_tableWidget.addColumn("timezone", dgettext('plugin_spreed', "Timezone"), 120, 6);
	this.participants_tableWidget.addColumn("status", "", 42, 7);

    this.participants_tableWidget.generateTable();
    this.participants_tableWidget.addRowListener(this.eventRowSelection, "all", this);

    // The table has changed, resize all the boxes accordingly.
    this.resize();
}

/**
 * This function processes all the events where the user
 * changed the selection or clicked in the participants
 * table. This is used to include the user in the web
 * meeting if it had been deselected earlier, this only
 * happens once the user clicks that entry.
 * @param TableWidget widget The TableWidget instance.
 * @param string type The type of event that was fired.
 * @param int[] selectIDs A list with ids of the entries
 * that were selected or deselected because of this event.
 * @param int[] changedIDs A list with ids of the entries
 * that changed because of this event.
 */
spreeditemmodule.prototype.eventRowSelection = function(widget, type, selectedIDs, changedIDs)
{
    switch(type){
        case "dblclick":
            // TODO Implement edit feature here
            break;
        case "select":
            if(this.selectCheckIsBusy || changedIDs.length > 1) break;

            document.getElementById('inc_' + changedIDs[0]).checked = 'checked';
            break;
        case "deselect":
            break;
    }
}

/**
 * This function will loop through the selected rows in
 * the participants table and select all the participants
 * that have a checked check-box. Although this selection is 
 * not visible to the user, this makes it easier to retrieve
 * all the data that belongs to the participants that need to
 * be included in the Spreed web meeting.
 * @param int clickedOnRowID The row id of the row that was
 * clicked, if this occured. Otherwise this value should be
 * set to 0.
 */
spreeditemmodule.prototype.selectCheckedRows = function(clickedOnRowID)
{
    if(this.selectCheckIsBusy) return;
    this.selectCheckIsBusy = true;

    // Clear the selection first
    this.participants_tableWidget.clearSelection();

    // Loop through the table and only include the rows that have a checked
    // include participant box.
    var table = dhtml.getElementById('tablewidget['+this.participants_tableWidget.widgetID+'][contenttable]');
    for(var i = 0; i < table.rows.length; i++){
        var r = table.rows[i];
        if(r.rowID == clickedOnRowID){
            if(dhtml.getElementById('inc_' + r.rowID).checked){
                this.participants_tableWidget.deselectRow(clickedOnRowID);
            }
        } else {
            if(dhtml.getElementById('inc_' + r.rowID).checked){
                if(!this.participants_tableWidget.isRowIDSelected(r.rowID)){
                    this.participants_tableWidget.selectRow(r.rowID);
                }
            } else {
                if(this.participants_tableWidget.isRowIDSelected(r.rowID)){
                    this.participants_tableWidget.deselectRow(r.rowID);
                }
            }
        }
    }

    this.selectCheckIsBusy = false;
}

/**
 * In order to check if a certain user is already in the participant
 * list, this function will determine what the row id is that represents
 * the user with that email address. If there is no such row in the table
 * it will return false.
 * @param string email The email address to look for.
 * @return int|boolean The row id as an integer, or false if the row
 * was not found.
 */
spreeditemmodule.prototype.getRowIDByEmail = function(email)
{
    var rowID = this.participants_tableWidget.nextRowID(0);
    while(rowID !== false){
        if(this.participants_tableWidget.getDataByRowID(rowID).email.innerHTML == escapeHtml(email))
            return rowID;

        rowID = this.participants_tableWidget.nextRowID(rowID);
    }
    return false;
}

/**
 * This function wraps the addParticipantEmail function, such that
 * in case you want to add multiple email addresses at the same
 * time this function can be called with an array to those email
 * addresses.
 * @param string[] vals The list of email addresses to process.
 * @see addParticipantEmail
 */
spreeditemmodule.prototype.addParticipantEmails = function(vals)
{
    var i;
    for(i = 0; i < vals.length; i++){
        this.addParticipantEmail(vals[i]);
    }
}

/**
 * Add a new participant to the table of participants based on
 * the supplied email address. This will call the
 * extractNameArrayFromEmailHeader function to extract the
 * names and organisation based on the email address.
 * @param string val The email address string.
 * @see extractNameArrayFromEmailHeader
 */
spreeditemmodule.prototype.addParticipantEmail = function(val)
{
    val = val.trim();
    if(val.length == 0)
        return;

    if(val == '[Everyone]'){
        alert(dgettext('plugin_spreed', "Unfortunately you cannot add everyone to your Spreed meeting, if you want to add groups please use Distribution Lists instead")+'.');
        return;
    }

    var par = Pluginspreed.prototype.extractNameArrayFromEmailHeader(val);
    this.addParticipant(
                par["firstname"],
                par["lastname"],
                par["organisation"],
                par["email"],
                true
            );
}

/**
 * This function will process the input specified by the user fields
 * underneath the participants table. The function will only proceed
 * adding the user if any of the following fields are left non-empty:
 *  - first name
 *  - last name
 *  - email
 * @param int ask Set this to 1 or 2 to ask the user whether the user should
 * be added. This is used with a call to this function before setting
 * up the meeting. If no fields were filled in it will just return,
 * but if any of the above mentioned fields were filled it will ask
 * the user what to do.
 * If the ask value is set to 1 then the question raised will be related
 * to the start of the meeting.
 * If the ask value is set to 2 then the question raised will be related
 * to the edit of a participant that might erase the current filled in
 * add fields.
 * @return boolean The function will return true if all went well or the user
 * decided that it is okey to continue. In case the user needs to
 * specify more information the function will return with false.
 * With this return value the calling setup meeting function is able
 * to quit so the user can correct the missing information.
 *
 * Scenarios:
 *      First name is specified, email is not: returns false if the
 *      user wants to add the participant.
 *
 *      Email is specified, first name and last name are not: returns
 *      false if the user still wants to add the participant.
 *
 *      First name and Email are correct: Returns true in all cases.
 *      
 *      No fields were specified: Returns true
 *
 **/
spreeditemmodule.prototype.addParticipantInput = function(ask)
{
    var rowID, nwLanguage, nwTimezone, languageOpt, timezoneOpt;
    var flds = Array();
    flds['newfirstname'] = dhtml.getElementById("newfirstname");
    flds['newlastname'] = dhtml.getElementById("newlastname");
    flds['neworganisation'] = dhtml.getElementById("neworganisation");
    flds['newemail'] = dhtml.getElementById("newemail");
    flds['newstatus'] = dhtml.getElementById("newstatus");
    timezoneOpt = dhtml.getElementById("newtimezone");
    languageOpt = dhtml.getElementById("newlanguage");
    nwLanguage = languageOpt.options[languageOpt.selectedIndex].value;
    nwTimezone = timezoneOpt.options[timezoneOpt.selectedIndex].value;

    if(flds['newfirstname'].value.length == 0 && flds['newlastname'].value.length == 0 && flds['newemail'].value.length == 0) // This ignores the organisation, language and timezone values. These are not important to check for the constraint
        return true; // Return true, no reason to add anything

    // Check if we should ask the user if (s)he wants to add the new user first
    if(ask == 1){
        if(!confirm(dgettext('plugin_spreed', "You are about to start your meeting, however, you didn't add %s to the meeting yet, shall I add it for you?").sprintf(escapeJavascript(flds['newfirstname'].value.length == 0 && flds['newlastname'].value.length == 0 ? flds['newemail'].value : (flds['newfirstname'].value + " " + flds['newlastname'].value))))){
            return true; // Return true, because this is not an error but a decision
        }
    } else if(ask == 2){
        return confirm(dgettext('plugin_spreed', "You are about to edit a user, however, you didn't add %s to the meeting yet. Are you sure you want to skip adding this user to the meeting?").sprintf(escapeJavascript(flds['newfirstname'].value.length == 0 && flds['newlastname'].value.length == 0 ? flds['newemail'].value : (flds['newfirstname'].value + " " + flds['newlastname'].value))));
    }

    // Is a valid email address supplied to us? If not warn the user.
    if(flds['newemail'].value.length == 0 || validateEmailAddress(flds['newemail'].value, true, true) == false){
        alert(dgettext('plugin_spreed', "Please specify a correct email address for this user") + ".");
        return false; // Return false on errors
    }

    // Are the firstname and lastname values specified? One of the two is enough
    if(flds['newfirstname'].value.length == 0 && flds['newlastname'].value.length == 0){
        var par = Pluginspreed.prototype.extractNameArrayFromEmailHeader(flds['newemail'].value);
        if(confirm(
                    dgettext('plugin_spreed', "You did not specify the first name and last name of the user, I could set it for you, are these fields correct?") + "\n"
                    + dgettext('plugin_spreed', "Firstname") + ": " + escapeJavascript(par['firstname']) + "\n"
                    + dgettext('plugin_spreed', "Surname") + ": " + escapeJavascript(par['lastname'])
                    + (flds['neworganisation'].value.length == 0 ? "\n" + dgettext('plugin_spreed', "Organisation") + ": " + escapeJavascript(par['organisation']) : '')
                  )){
            // The user accepts our name suggestions, process these first.
            // We do not have to process the escapeHtml here, because it will be processed in the addParticipant function.
            flds["newfirstname"].value = par['firstname'];
            flds["newlastname"].value = par['lastname'];
            flds["neworganisation"].value = par['organisation'];
        } else {
            // The user wants to correct it manually, make sure the calling functions stops processing
            return false;
        }
    }

    // Call the addParticipant function to add it officially
    this.addParticipant(
                flds["newfirstname"].value,
                flds["newlastname"].value,
                flds["neworganisation"].value,
                flds["newemail"].value,
                true,
                nwLanguage,
                nwTimezone
            );

    // Clear all the input fields
    flds["newfirstname"].value = '';
    flds["newlastname"].value = '';
    flds["neworganisation"].value = '';
    flds["newemail"].value = '';

    flds["newfirstname"].focus();

    return true; // Return true, everything went well, user is added
}

/**
 * Process the server responce we got after we asked it to retrieve
 * all the participants from the selected emails or contacts.
 * @param xml list The xml list of all the participants to add.
 * @see addParticipant
 */
spreeditemmodule.prototype.addParticipantXMLList = function(list)
{
    var i;
    for(i = 0; i < list.length; i++){
        this.addParticipant(
                    dhtml.getXMLValue(list[i], "firstname"),
                    dhtml.getXMLValue(list[i], "lastname"),
                    dhtml.getXMLValue(list[i], "organisation"),
                    dhtml.getXMLValue(list[i], "email"),
                    dhtml.getXMLValue(list[i], "alternative") ? false : true
                );
    }
}

/**
 * Add multiple participants from a list of name blocks.
 * @param string[] list The list with the name fields.
 * @see addParticipant
 */
spreeditemmodule.prototype.addParticipants = function(list)
{
    var i;
    for(i = 0; i < list.length; i++){
        this.addParticipant(
                    list[i]["firstname"],
                    list[i]["lastname"],
                    list[i]["organisation"],
                    list[i]["email"], 
                    list[i]["alternative"] ? false : true
                );
    }
}

/**
 * Check whether the participant with the given firstname,
 * surname, organisation and email exist.
 * At this moment the function will determine if it exists
 * solely on the email address.
 * @param string firstname The firstname of the participant
 * @param string lastname The surname of the participant
 * @param string organisation The organisation of the participant
 * @param string email The email address of the participant
 * @return boolean True if the participant exists, false
 * otherwise.
 */
spreeditemmodule.prototype.participantExists = function(firstname, lastname, organisation, email)
{
    var table = dhtml.getElementById('tablewidget['+this.participants_tableWidget.widgetID+'][contenttable]');
    for(var i = 0; i < table.rows.length; i++){
        var r = table.rows[i];
        var data = this.participants_tableWidget.getDataByRowID(r.rowID);
        if(escapeHtml(email) == data.email.innerHTML)
            return true;
    }
    return false;
}

/**
 * Add the participant with the given information
 * to the participants table.
 * @param string firstname The firstname of the participant
 * @param string lastname The lastname of the participant
 * @param string organisation The organisation of the participant
 * @param string email The email address of the participant
 * @param string language The language in which the emails should
 * be sent to the participant.
 * @param string timezone The timezone of the participant
 * @return int The row id of the row that was inserted in the
 * participants table.
 */
spreeditemmodule.prototype.addParticipant = function(firstname, lastname, organisation, email, checked, language, timezone)
{
    var i, checkboxElem, statusElem, newStatusElem, editElem, origTimezone, nwTimezone, nwLanguage, row;
    if(this.participantExists(firstname, lastname, organisation, email))
        return;
    //var emailID = this.getRowIDByEmail(email);
    //if(emailID !== false){
    //    alert("This user already exists in your conference: " + email);
    //    return;
    //}

    var rowID = this.participants_tableWidget.addRow(
            {
				include: {
					innerHTML: ""
				},
				firstname: {
					innerHTML: escapeHtml(firstname)
				},
				lastname: {
					innerHTML: escapeHtml(lastname)
				},
				organisation: {
					innerHTML: escapeHtml(organisation)
				},
				email: {
					innerHTML: escapeHtml(email)
				},
				language: {
					innerHTML: ""
				},
				timezone: {
					innerHTML: ""
				},
                status: {
                    innerHTML: ""
                }
			});


    checkboxElem = document.createElement("input");
    checkboxElem.setAttribute("type", 'checkbox');
    checkboxElem.setAttribute("id", 'inc_' + rowID);
    checkboxElem.setAttribute("name", 'inc_' + rowID);
    if(checked)
        checkboxElem.setAttribute("checked", 'checked');
    checkboxElem.setAttribute("value", 'true');
    checkboxElem.setAttribute("title", dgettext('plugin_spreed', "Invite"));
    checkboxElem.setAttribute("onMouseDown", 'module.selectCheckIsBusy = true;');
    checkboxElem.setAttribute("onClick", 'module.includeBoxClicked(' + rowID + ');');
    checkboxElem.setAttribute("onMouseUp", 'module.selectCheckIsBusy = false;');

    // Used to copy the moderator / user type details
    newStatusElem = dhtml.getElementById('newstatus');

    statusElem = document.createElement("div");
    statusElem.setAttribute("class", newStatusElem.getAttribute('class'));
    statusElem.setAttribute("id", 'st_' + rowID);
    statusElem.setAttribute("title", newStatusElem.getAttribute('title'));
    statusElem.setAttribute("alt", newStatusElem.getAttribute('alt'));
    statusElem.setAttribute("onClick", 'module.statusClicked(' + rowID + ');');

    editElem = document.createElement("div");
    editElem.setAttribute("class", 'icon_spreed_edit');
    editElem.setAttribute("id", 'e_' + rowID);
    editElem.setAttribute("title", dgettext('plugin_spreed', "Edit participant"));
    editElem.setAttribute("alt", dgettext('plugin_spreed', "Edit"));
    editElem.setAttribute("onClick", 'module.editClicked(' + rowID + ');');

    origTimezone = dhtml.getElementById("timezone");
    origLanguage = dhtml.getElementById("newlanguage");

    // Clone the timezone select box
    nwTimezone = this.makeSelectClone(origTimezone, 'tz_' + rowID, timezone);
    dhtml.addEvent(this, nwTimezone, "mousedown", spreedCancelEventPropagation);

    // Clone the language select box
    nwLanguage = this.makeSelectClone(origLanguage, 'l_' + rowID, language);
    dhtml.addEvent(this, nwLanguage, "mousedown", spreedCancelEventPropagation);

    // Obtain the row element
    row = this.participants_tableWidget.getRowByRowID(rowID);
    dhtml.removeEvents(row);

    // Insert the checkbox
    row.childNodes[0].appendChild(checkboxElem);

    // Insert the language
    row.childNodes[5].appendChild(nwLanguage);

    // Insert the timezone
    row.childNodes[6].appendChild(nwTimezone);

    // Insert the user type indicator and edit button
    row.childNodes[7].appendChild(statusElem);
    row.childNodes[7].appendChild(editElem);

    // By default the box is checked, so make the row appear selected as well
    if(checked)
        this.participants_tableWidget.selectRow(rowID);

    // Display a warning message if necessary
    // This will also resize all components
    this.updateParticipantWarningMessage();

    return rowID;
}

/**
 * This function clones the given select box with the given option set.
 * @param htmlelement orig The original html element of the select
 * box that you want to copy
 * @param string id The new id of the clone.
 * @param string sel The value that should be selected, or null in
 * case you want to copy the selection of the original select box
 * as well.
 * @return htmlelement The htmlelement of the cloned select box.
 */
spreeditemmodule.prototype.makeSelectClone = function(orig, id, sel)
{
    if(sel == null){
        sel = orig.options[orig.selectedIndex].value;
    }

    var cloned = orig.cloneNode(true);
    cloned.setAttribute("onChange", "");
    cloned.setAttribute("id", id);
    cloned.setAttribute("name", id);

    this.selectInOption(cloned, sel);

    return cloned;
}

/**
 * This function will select the supplied option in the
 * select box element given. This is used to copy the
 * selection from the timezone field of the meeting to the
 * new participant timezone field at the middle of the dialog.
 * @param htmlelement elem The select box element where the
 * selection should be changed.
 * @param string sel The option that should be selected in the
 * selection box.
 */
spreeditemmodule.prototype.selectInOption = function(elem, sel)
{
    for(var i = 0; i < elem.options.length; i++){
        elem.options[i].selected = (sel == elem.options[i].value);
    }
}

/**
 * This function processes the event in which the user changed
 * the main timezone of the Spreed web meeting, in this case the
 * new value should be copied to the timezone of the user and the
 * timezone of all new participants that are added.
 */
spreeditemmodule.prototype.timezoneChanged = function(){
    var origTimezone = dhtml.getElementById("timezone");
    var addTimezone = dhtml.getElementById("newtimezone");
    var myTimezone = dhtml.getElementById("tz_" + this.creatorRowID);
    var timezone = origTimezone.options[origTimezone.selectedIndex].value;
    this.selectInOption(addTimezone, timezone);
    this.selectInOption(myTimezone, timezone);
}

/**
 * This function process the click event of the user, where he or she
 * clicked on a include check box of an entry in the participant table.
 * This will make sure that the text is grayed out if the user is not
 * selected any more, or is changed back to black if it is.
 * @param int rowID The row id of the row where the user clicked on.
 */
spreeditemmodule.prototype.includeBoxClicked = function(rowID)
{
    var optionElem = dhtml.getElementById('inc_' + rowID);
    if(!optionElem.checked){
        dhtml.addClassName(optionElem.parentNode.parentNode, 'dis');
    } else {
        dhtml.removeClassName(optionElem.parentNode.parentNode, 'dis');
    }

    // Display the warning message if necessary, or remove
    // it if the user selected enough participants.
    this.updateParticipantWarningMessage();
}

/**
 * Once the user decides to promote a user from normal user to moderator,
 * or when the user wants to demote the moderator back to a normal user,
 * this function is called. This will change the icon and make sure the
 * user gets the right flags in the participant table.
 * @param int rowID The row id of the entry which the user wants to pro-
 * mote or demote.
 */
spreeditemmodule.prototype.statusClicked = function(rowID)
{
    var statusElem = rowID > 0 ? dhtml.getElementById('st_' + rowID) : dhtml.getElementById('newstatus');
    statusElem.setAttribute("title", statusElem.className == 'icon_spreed_user' ?  dgettext('plugin_spreed', "Demote to user") : dgettext('plugin_spreed', "Promote to moderator"));
    statusElem.setAttribute("alt", statusElem.className == 'icon_spreed_user' ? 'M' : 'U');
    statusElem.className = statusElem.className == 'icon_spreed_user' ? 'icon_spreed_mod' : 'icon_spreed_user'; 
}

/**
 * In case the user clicks the edit button in the participant table
 * this function is called. This will remove the row from the paricipant
 * table and put the fields in the add new participant fields.
 * If, however, the user filled some of the new participant fields, the
 * user will be asked if the data should be flushed or if the user should
 * be added to the participant table first, before we are able to edit it.
 * 
 * @param int rowID The row id which the user wants to add.
 */
spreeditemmodule.prototype.editClicked = function(rowID)
{
    var i, tzElem, lElem, curStElem, stElem, rowData, row, nextRow;

    // Call the addParticipantInput function with ask set to true, this
    // will ask the user if (s)he didn't finish adding the user to the
    // participant table (either the firstname, lastname or email fields
    // are non-empty). 
    if(this.addParticipantInput(2) == false){
        // False is returned in case the user needs to correct the email
        // address or specify the firstname or lastname first.
        // In this case we should stop setting-up the meeting.
        return;
    }

    // Set all the fields for adding a new participant with the selected participant
    rowData = this.participants_tableWidget.getDataByRowID(rowID);
    dhtml.getElementById("newfirstname").value = rowData.firstname.innerHTML;
    dhtml.getElementById("newlastname").value = rowData.lastname.innerHTML;
    dhtml.getElementById("neworganisation").value = rowData.organisation.innerHTML;
    dhtml.getElementById("newemail").value = rowData.email.innerHTML;

    // Get the user type indicators (Moderator / User)
    curStElem = dhtml.getElementById("st_" + rowID);
    stElem = dhtml.getElementById('newstatus');

    // Copy the current setting to the add user button
    stElem.setAttribute("class", curStElem.getAttribute("class"));
    stElem.setAttribute("title", curStElem.getAttribute("title"));
    stElem.setAttribute("alt", curStElem.getAttribute("alt"));

    tzElem = dhtml.getElementById("tz_" + rowID);
    lElem = dhtml.getElementById("l_" + rowID);

    this.selectInOption(dhtml.getElementById("newtimezone"), tzElem.options[tzElem.selectedIndex].value);
    this.selectInOption(dhtml.getElementById("newlanguage"), lElem.options[lElem.selectedIndex].value);

    // Make sure the styles set for this row will be copied to the next row
    // This is necessary since the width is defined in the style field of the
    // first row only, if we delete this row the other rows will be rendered
    // differently. Thus, by copying the style attributes of the to-be-deleted
    // row to the next row in the table this problem is solved.
    row = this.participants_tableWidget.getRowByRowID(rowID);
    if(row && row.nextSibling && row.nextSibling.getAttribute){
        nextRow = row.nextSibling;
        // Copy for all its child nodes:
        for(i = 0; i < nextRow.childNodes.length; i++){
            if(row.childNodes[i] && row.childNodes[i].getAttribute && nextRow.childNodes[i].getAttribute){
                nextRow.childNodes[i].setAttribute('style', row.childNodes[i].getAttribute('style'));
            }
        }
    }

    // Remove the row from the table
    // We need to delete this manually from the html table
    // and after that from the data structure, because otherwise
    // the tablewidget will redraw the table which causes all our
    // fields to be lost.
    this.participants_tableWidget.deleteRowHTML(rowID);
    this.participants_tableWidget.deleteData(rowID);

    // Make sure all the rows are selected which will be included in the meeting
    this.selectCheckedRows();

    // Set the focus to the firstname field
    dhtml.getElementById("newfirstname").focus();
}


/**
* Make a XML request to get all the contact, mail, or distlist data
* @param data must be an array with the store and folder entryids
* For example:
* [{'store': 'x', 'entryid': 'x'}, {'store': 'y', 'entryid': 'y'}]
* @param type The type of the request to ask to the server, either:
* 'getMail', 'getContact', or 'getDistList'
*/
spreeditemmodule.prototype.getEntriesFromServer = function(data, type)
{
    var i;
    var emData;
    for(i = 0; i < data.length; i++){
        emData = Array();
        emData['entryid'] = data[i]['entryid'];
        emData['store'] = data[i]['store'];
        if(type == 'getMail')
            emData['dialog_attachments'] = dhtml.getElementById("dialog_attachments").value;
        webclient.xmlrequest.addData(this, type, emData);
    }
	webclient.xmlrequest.sendRequest();
}

/**
* Make a XML request to setup the Meeting in Spreed
* @param data must be an array with the title, description
* For example:
* {
*   'title': 'The conference title',
*   'description': 'The conference description'
* }
*/
spreeditemmodule.prototype.setupCall = function()
{
    // Call the addParticipantInput function with ask set to true, this
    // will ask the user if (s)he didn't finish adding the user to the
    // participant table (either the firstname, lastname or email fields
    // are non-empty). Otherwise the 
    if(this.addParticipantInput(1) == false){
        // False is returned in case the user needs to correct the email
        // address or specify the firstname or lastname first.
        // In this case we should stop setting-up the meeting.
        return;
    }

    this.selectCheckedRows();
    var nrOfParticipants = this.participants_tableWidget.getSelectedRowData().length;
    if(nrOfParticipants == 1){ // This number is set to one because it will count the user as one.
        alert(dgettext('plugin_spreed', "You need at least one other participant to set-up a Spreed Meeting, select or add another participant to continue") + ".");
    } else if(nrOfParticipants > 3){ // This number is set to three because it will also count the creator.
        alert(dgettext('plugin_spreed', "Too many participants. You are not allowed to add more than two other participants with the free version of Spreed") + ".");
    } else {
        dhtml.removeClassName(dhtml.getElementById('overlay'), 'hidden');
        webclient.xmlrequest.addData(this, "setupCall", this.constructConferencePayload());
        webclient.xmlrequest.sendRequest();
    }
}

/**
 * This function will open the attachment dialog for
 * this Spreed web meeting.
 */
spreeditemmodule.prototype.openAttachmentDialog = function()
{
    webclient.openWindow(this, 'attachments', DIALOG_URL+'task=attachments_modal&store=' + this.storeid + '&entryid=&dialog_attachments=' + dhtml.getElementById('dialog_attachments').value, 570, 425, '0');
}

/**
 * This function will open the address book dialog for
 * this Spreed web meeting.
 */
spreeditemmodule.prototype.openAddressBook = function()
{
    webclient.openModalDialog(this, 'addressbook', DIALOG_URL+'task=addressbook_modal&storeid=' + this.storeid, 800, 500, abCallBackSpreed);
}

/**
 * This function constructs the payload XML data that is
 * sent to the server to set-up the Spreed web meeting.
 * @return xml The XML payload instance of the web meeting.
 */
spreeditemmodule.prototype.constructConferencePayload = function()
{
    var tzFld = dhtml.getElementById("timezone");
    return {
        'title': dhtml.getElementById("title").value,
        'description': dhtml.getElementById("description").value,
        'timezone': tzFld.options[tzFld.selectedIndex].value,
        'start_time': this.combinedTime.getStartValue(),
        'duration': (this.combinedTime.getEndValue() - this.combinedTime.getStartValue()) / 60,
        'dialog_attachments': dhtml.getElementById("dialog_attachments").value,
        'participants': {
            'attributes': {'id': 'list'},
            'user': this.constructParticipantPayload()
        }
    };
}

/**
 * This function constructs the payload XML data for all the
 * participants of the spreed meeting.
 * @return xml The XML payload instance of the participants in
 * this meeting.
 */
spreeditemmodule.prototype.constructParticipantPayload = function()
{
    var r, participant, tz, lang, mod;
    var retPayload = Array();
    var selRowData= this.participants_tableWidget.getSelectedRowData();
    for(r = 0; r < selRowData.length; r++)
    {
        // We do not need to add ourselfs to the participant payload
        if(this.creatorRowID == selRowData[r]['rowID'])
            continue;

        tz = dhtml.getElementById('tz_' + selRowData[r]['rowID']);
        lang = dhtml.getElementById('l_' + selRowData[r]['rowID']);
        mod = dhtml.getElementById('st_' + selRowData[r]['rowID']);
        participant = Array();
        participant['attributes'] = {'id': r};
        participant['firstname'] = selRowData[r].firstname.innerHTML;  
        participant['lastname'] = selRowData[r].lastname.innerHTML;  
        participant['organisation'] = selRowData[r].organisation.innerHTML;  
        participant['email'] = selRowData[r].email.innerHTML;  
        participant['language'] = lang.options[lang.selectedIndex].value;
        participant['timezone'] = tz.options[tz.selectedIndex].value;
        participant['type'] = mod.getAttribute('alt');
        retPayload.push(participant);
    }
    return retPayload;
}

/**
 * If the user decides to clear the Spreed web meeting
 * dialog this method is called. This will erase all the
 * fields in the Spreed dialog, except the first row entry
 * of the user.
 */
spreeditemmodule.prototype.clear = function()
{
    this.participants_tableWidget.resetWidget();
    this.setupParticipantsTable();
    this.addCreatorToTable(false);

    // Display the warning message, because now the user
    // is the only participant in the table.
    this.updateParticipantWarningMessage();

    dhtml.getElementById("title").value = '';
    dhtml.getElementById("description").value = '';
	module.setAttachmentData(Array());
	module.setAttachments();
    webclient.xmlrequest.addData(this, "clear", {
        'dialog_attachments': dhtml.getElementById("dialog_attachments").value
    });
    webclient.xmlrequest.sendRequest();
}

/**
 * Override the default addAttachments function, this allows us to display
 * a warning message, or hide the warning message depending on the files that
 * the user wants to include in the Spreed web meeting.
 *
 * @param DHTMLElement attachmentsElement The attachment dhtml element.
 * @param Attachment[] attachments The array of attachment structures.
 **/
spreeditemmodule.prototype.addAttachments = function(attachmentsElement, attachments)
{
    // Show or hide the warnings
    this.updateAttachmentWarningMessage(attachments);

    // Process the attachment with the overridden method
	return spreeditemmodule.superclass.addAttachments.call(this, attachmentsElement, attachments);
}

/**
 * Update the warning messages according to the files that are being made
 * available to the participants of the Spreed web meeting. If the file is
 * going to be uploaded as an upload attachment than all users of the meeting
 * will be able to download the files. Presentation files will only be visible
 * to the users, but cannot be downloaded.
 *
 * @param Attachment[] attachments The array of attachment structures.
 **/
spreeditemmodule.prototype.updateAttachmentWarningMessage = function(attachments)
{
    var warningElement = dhtml.getElementById('uploadWarning');
    var nonPresentationFiles = Array();

    // Loop through the attachments to check the file types
    if(attachments && attachments.length > 0){
        for(var i = 0; i < attachments.length; i++){
            var attach = attachments[i];

            // Check whether this is file is a presentation, if not add it to the list
            if(attach["attach_num"] && attach["name"] && this.isPresentationType(attach["name"]) == false){
                nonPresentationFiles.push(attach["name"]);
            }
        }
    }

    // Check if the user uploaded any files that cannot be
    // uploaded as a presentation in the Spreed web meeting.
    if(nonPresentationFiles.length > 0){
        // There are files to warn about
        warningElement.innerHTML = "<span>" + (nonPresentationFiles.length == 1
                                        ? dgettext('plugin_spreed', 'Please note that the following file cannot be uploaded as a presentation file. Therefore it will be accessible as a download to the participants of the web meeting.') + "<br />" + dgettext('plugin_spreed', 'Downloadable file') + ": "
                                        : dgettext('plugin_spreed', 'Please note that the following files cannot be uploaded as presentation files. Therefore they will be accessible as downloads to the participants of the web meeting.') + "<br />" + dgettext('plugin_spreed', 'Downloadable files') + ": ");

        // List the files and show the warning message
        for(var i = 0; i < nonPresentationFiles.length; i++){
            warningElement.innerHTML += (i > 0 ? ', ' : '') + escapeHtml(nonPresentationFiles[i]);
        }
        warningElement.innerHTML += "</span>";
        dhtml.removeClassName(warningElement, 'hidden');
    } else {
        // All files (if any were uploaded) can be uploaded
        // as presentation files. Therefore, hide the warning
        // if it wasn't already hidden.
        dhtml.addClassName(warningElement, 'hidden');
    }

    // Since the warning could have been added or removed we need
    // to resize the other components accordingly.
    this.resize();
}

/**
 * Update the warning messages according to the participants that are going
 * to be invited to the Spreed web meeting. If there are more than three
 * participants selected in the meeting, it will show a warning message to
 * the user.
 **/
spreeditemmodule.prototype.updateParticipantWarningMessage = function()
{
    var warningElement = dhtml.getElementById('participantWarning');
    var setupElement = dhtml.getElementById('spreed_call');
    this.selectCheckedRows();
    var nrOfParticipants = this.participants_tableWidget.getSelectedRowData().length;
    if(nrOfParticipants == 1){ // This number is set to one because it will count the user as one.
        warningElement.innerHTML = "<span>" + dgettext('plugin_spreed', "You need at least one other participant to set-up a Spreed Meeting, select or add another participant to continue") + ".</span>";
        dhtml.removeClassName(warningElement, 'hidden');
        dhtml.addClassName(setupElement, 'icon_spreed_call_dis');
    } else if(nrOfParticipants > 3){ // This number is set to three because it will also count the creator.
        warningElement.innerHTML = "<p>" + dgettext('plugin_spreed', "At the moment you have too many participants selected. In total you allowed to add three participants to the web meeting, including yourself. You have %s participants selected, please deselect atleast %d participants in order to setup the Spreed web meeting.").sprintf(nrOfParticipants, nrOfParticipants - 3) + "</span>";
        dhtml.removeClassName(warningElement, 'hidden');
        dhtml.addClassName(setupElement, 'icon_spreed_call_dis');
    } else {
        // No warnings to be shown, hide the warning message
        // if it wasn't already hidden.
        dhtml.addClassName(warningElement, 'hidden');
        dhtml.removeClassName(setupElement, 'icon_spreed_call_dis');
    }

    // Resize all the components
    this.resize();
}

/**
 * Checks whether the given filename is supported by Spreed to
 * be shown in Presentation mode. All other files will be uploaded
 * as attachments of the Spreed meeting. Attachments of web meetings
 * are downloadable for all the participants.
 *
 * @param string filename The file name of the attachment that was
 * just added to the web meeting.
 * @return boolean true if the file type is supported by Spreed to
 * be shown as a presentation. Returns false otherwise.
 */
spreeditemmodule.prototype.isPresentationType = function(filename)
{
    var m = filename.match(/\.(\w+)$/);
    switch(m[1]){
        /* Intentional fallthrough for all accepted files */
        case 'ppt':      // Microsoft Powerpoint
        case 'pptx':     // Microsoft Powerpoint XML
        case 'pps':      // Microsoft Powerpoint
        case 'doc':      // Microsoft Word
        case 'docx':     // Microsoft Word XML
        case 'xls':      // Microsoft Excel Spreadsheet
        case 'xlt':      // Microsoft Excel Spreadsheet
        case 'xlm':      // Microsoft Excel Spreadsheet
        case 'xld':      // Microsoft Excel Spreadsheet
        case 'xla':      // Microsoft Excel Spreadsheet
        case 'xlc':      // Microsoft Excel Spreadsheet
        case 'xlw':      // Microsoft Excel Spreadsheet
        case 'xll':      // Microsoft Excel Spreadsheet
        case 'odp':      // Open office presentation
        case 'odt':      // Open office document
        case 'ods':      // Open office spreadsheet
        case 'eps':      // Enhanced Postscript file
        case 'ps':       // Postscript file
        case 'pdf':      // Portable Document Format
        case 'rtf':      // General Rich Text Format
        case 'txt':      // General Plain Text File
        case 'png':      // Image
        case 'jpe':      // Image
        case 'jpg':      // Image
        case 'jpeg':     // Image
        case 'gif':      // Image
        case 'bmp':      // Image
        case 'ico':      // Image
        case 'tiff':     // Image
        case 'tif':      // Image
        case 'svg':      // Image
        case 'svgz':     // Image
        case 'mpeg':     // Movie
        case 'mpg':      // Movie
        case 'mpe':      // Movie
        case 'avi':      // Movie
        case 'mov':      // Movie
            return true;
        default:
            return false;
    }
}

/**
 * Function which adds data in the dialogbox fields.
 * @param object action the action tag 
 */ 
spreeditemmodule.prototype.item = function(action) {
}


/**
 * Function resize the dialog box elements like directionpane and map element when dialogbox is resized
 */
spreeditemmodule.prototype.resize = function()
{
	// Fetch the height of window title element.
	//title_element_height = dhtml.getElementById("windowtitle").offsetHeight;

	// Fetch the height of subtitle element.
	//subtitle_element_height = dhtml.getElementById("subtitle").offsetHeight;
    dhtml.getElementById('newparticipant').setAttribute('style', 'width: ' + dhtml.getElementById('tableWidgetContainer').offsetWidth + 'px');
    dhtml.getElementById('description').setAttribute('style', 'height: ' + (window.document.body.offsetHeight - dhtml.getElementById('attachmentbox').offsetTop - dhtml.getElementById('attachmentbox').offsetHeight - 22) + 'px');
}

/**
 * If the events need to be stopped propagating downwards you
 * can add this function to the event listener.
 * This is added because browsers like Safari process the mouse
 * down event populating downwards, and kill the event if one
 * of the lower layers returns with false. This results selection
 * boxes to be non-clickable because the event is caught inside the
 * table widget.
 * 
 * @param Module module The module of the calling element
 * @param DHTMLElement The element upon which the event was fired.
 * @param Event e The event object that describes which event occured.
 * @return boolean true in all cases, so the element is allowed to
 * process the event.
 **/
function spreedCancelEventPropagation(module, element, e){
    e = fixEvent(e);
    e.stopPropagation();

    return true;
}
